function [ TwoLS , est_ratio] = TwoLoopSeriesMCMC(edgeStruct, nodeBelLBP, edgeBelLBP,cooling_length, sample_num,sample_num2, burnin1, burnin2)
nEdges = double(edgeStruct.nEdges);
edgeEnds = double(edgeStruct.edgeEnds);
nNodes = double(edgeStruct.nNodes);
nComponents = double(edgeStruct.nComponents);

% compute edge weights based on BP fixed points
edgeWeights = zeros(nEdges,1);
for e = 1:nEdges
    tau_u = nodeBelLBP(edgeEnds(e,1),2);
    tau_v = nodeBelLBP(edgeEnds(e,2),2);
    tau_uv = edgeBelLBP(2,2,e);
    edgeWeights(e,1) =  abs((tau_uv - tau_u*tau_v) / (tau_u *tau_v *(1-tau_u) * (1-tau_v))^(0.5));
    if isnan(edgeWeights(e,1))
        edgeWeights(e,1) = 1;
    end
end

% run step 1 (estimating absolute loop series)
X =  zeros(nEdges,1);
approxProd= 2^(nEdges - nNodes + nComponents);
for i = 1: cooling_length
    beta = (i-1)/cooling_length;
    approxRatio = 0;
    for j = 1:sample_num
        W = edgeWeights.^beta;
        X = WormMC(W, edgeStruct, burnin1, X); % Markov Chain is defined by Worm algorithm
        approxRatio = approxRatio + (abs(LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, 1, X))^(1/cooling_length));
    end
    approxProd = approxProd * approxRatio/sample_num;
end

% run step 2 (estimating negative ratio)
minus_ratio = 0;
X = zeros(nEdges,1);
for j = 1:sample_num2
    X = WormMC(edgeWeights, edgeStruct, burnin2, X);
    if LoopTerm(edgeStruct,nodeBelLBP, edgeBelLBP, 1, X)< -1e-6
        minus_ratio = minus_ratio + 1;
    end
end
minus_ratio = minus_ratio / sample_num2;
est_ratio = 1- 2* minus_ratio;

%estimated ratio cannot be negative
if est_ratio < 1/sample_num2
    est_ratio = 1/sample_num2;
end

TwoLS = log((1-2*minus_ratio) * approxProd);
end